@@ -1,5 +1,5 @@
 from StructNoSQL.dynamodb.dynamodb_core import PrimaryIndex, GlobalSecondaryIndex
 from StructNoSQL.fields import BaseField, MapModel, MapField, TableDataModel
 from StructNoSQL.table import BaseTable
-from StructNoSQL.validator import NoneType
+from StructNoSQL.validator import NoneType, ActiveSelf
 from StructNoSQL.dynamodb.models import FieldGetter, FieldSetter, UnsafeFieldSetter, FieldRemover
