@@ -72,12 +72,11 @@ class BaseItem:
                 elif alias_variable_name == "List":
                     raise Exception(f"List not yet implemented.")
 
-    def validate_data(self, load_data_into_objects: bool = False) -> Tuple[Optional[Any], bool]:
+    def validate_data(self) -> Tuple[Optional[Any], bool]:
         from StructNoSQL.validator import validate_data
         validated_data, valid = validate_data(
-            value=self._value, item_type_to_return_to=self, load_data_into_objects=load_data_into_objects,
-            expected_value_type=self._field_type, map_model=self.map_model,
-            dict_items_excepted_type=self.dict_items_excepted_type, dict_excepted_key_type=self.dict_key_expected_type
+            value=self._value, item_type_to_return_to=self,
+            expected_value_type=self._field_type,
         )
         self._value = validated_data
         return validated_data, valid
@@ -148,7 +147,6 @@ class BaseItem:
     def database_path(self) -> Optional[List[DatabasePathElement]]:
         return self._database_path
 
-
     @property
     def table(self):
         return self._table
@@ -156,18 +154,39 @@ class BaseItem:
 
 class BaseField(BaseItem):
     def __init__(self, name: str, field_type: Optional[Any] = None, required: Optional[bool] = False, not_modifiable: Optional[bool] = False,
-                 custom_default_value: Optional[Any] = None, key_name: Optional[str] = None):
+                 custom_default_value: Optional[Any] = None, key_name: Optional[str] = None, max_nested_depth: Optional[int] = 32):
         super().__init__(field_type=field_type if field_type is not None else Any, custom_default_value=custom_default_value)
         self._name = name
         self._required = required
         self._key_name = None
 
+        if max_nested_depth is not None and max_nested_depth > 32:
+            raise Exception(f"DynamoDB support a maximum depth of nested of items of 32. This is not imposed by StructNoSQL but a platform limitation.\n"
+                            f"See : https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html#limits-attributes")
+        self._max_nested = max_nested_depth
+
         if not_modifiable is True:
             raise Exception(f"Not modifiable not yet implemented")
 
         if key_name is not None:
-            if field_type == dict or type(field_type) == _GenericAlias:
+            insta = isinstance(field_type, (tuple, list))
+            if field_type is dict or type(field_type) is _GenericAlias:
                 self._key_name = key_name
+            elif isinstance(field_type, (tuple, list)):
+                raise Exception(f"Multiple dictionaries are not yet supported.")
+                all_items_are_dict = True
+                for item in field_type:
+                    item_type = type(item)
+                    if not isinstance(item, (dict, _GenericAlias)):
+                        all_items_are_dict = False
+                        break
+                if all_items_are_dict is True:
+                    self._key_name = key_name
+                else:
+                    raise Exception(message_with_vars(
+                        "key_name cannot be set on a field that is a tuple or list that does not exclusivly contains dict or Dict items",
+                        vars_dict={"fieldName": name, "fieldType": field_type, "keyName": key_name}
+                    ))
             else:
                 raise Exception(message_with_vars(
                     "key_name cannot be set on a field that is not of type dict or Dict",
@@ -194,29 +213,6 @@ class BaseField(BaseItem):
                            "dictItemsExceptedType": self.dict_items_excepted_type}
             ))
 
-    def __getattr__(self, key):
-        item = self.__dict__.get(key, None)
-        if item is not None:
-            return item
-        else:
-            print(key)
-            print(self.dict_items_excepted_type.__dict__)
-            if self._field_type == dict:
-                if isinstance(self.dict_items_excepted_type, (list, tuple)):
-                    for expected_type in self.dict_items_excepted_type:
-                        dict_expected_type_item = expected_type.__dict__.get(key, None)
-                        if dict_expected_type_item is not None:
-                            return dict_expected_type_item
-                else:
-                    dict_expected_type_item = self.dict_items_excepted_type.__dict__.get(key, None)
-                    if dict_expected_type_item is not None:
-                        return dict_expected_type_item
-
-        raise AttributeError(message_with_vars(
-            message="Attribute missing from BaseField and its dict_item expected type.",
-            vars_dict={"attributeKey": key, "__dict__": self.__dict__}
-        ))
-
     def post(self, value: any):
         print(self._database_path)
 
@@ -232,6 +228,15 @@ class BaseField(BaseItem):
     def required(self) -> bool:
         return self._required
 
+    @property
+    def max_nested(self) -> int:
+        # todo: find a better name than max_nested
+        return self._max_nested
+
+    def copy(self):
+        # return BaseField(name=self._name, field_type=self._field_type, required=self._required, custom_default_value=self._custom_default_value, key_name=self._key_name, max_nested_depth=self._max_nested)
+        from copy import copy
+        return copy(self)
 
 class MapItem(BaseField):
     _default_primitive_type = dict
