@@ -1,8 +1,9 @@
 from typing import Optional, List, Dict, Any, Set
-from StructNoSQL.dynamodb.dynamodb_core import DynamoDbCoreAdapter, PrimaryIndex, GlobalSecondaryIndex, \
-    DynamoDBMapObjectSetter, Response
+from copy import copy
+
+from StructNoSQL.dynamodb.dynamodb_core import DynamoDbCoreAdapter, PrimaryIndex, GlobalSecondaryIndex, DynamoDBMapObjectSetter, Response
 from StructNoSQL.dynamodb.models import DatabasePathElement, FieldGetter, FieldSetter, UnsafeFieldSetter, FieldRemover
-from StructNoSQL.fields import BaseField, MapModel, MapField, MapItem, TableDataModel
+from StructNoSQL.fields import BaseField, MapField, MapItem, TableDataModel
 from StructNoSQL.practical_logger import message_with_vars
 from StructNoSQL.utils.process_render_fields_paths import process_and_get_fields_paths_objects_from_fields_paths, \
     process_and_make_single_rendered_database_path, process_validate_data_and_make_single_rendered_database_path, \
@@ -17,12 +18,26 @@ class DatabaseKey(str):
     pass
 
 
+class FieldsSwitch(dict):
+    def __init__(self, *args, **kwargs):
+        super(dict).__init__(*args, **kwargs)
+
+    def set(self, key: str, item: MapField or BaseField) -> bool:
+        if len(item.database_path) > 32:
+            print("\nDynamoDB support a maximum depth of nested of items of 32. This is not imposed by StructNoSQL but by DynamoDB.\n"
+                  "See : https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/Limits.html#limits-attributes")
+            return False
+        else:
+            self.__setitem__(key, item)
+            return True
+
+
 class BaseTable:
     def __init__(self, table_name: str, region_name: str, data_model, primary_index: PrimaryIndex,
                  create_table: bool = True, billing_mode: str = DynamoDbCoreAdapter.PAY_PER_REQUEST,
                  global_secondary_indexes: List[GlobalSecondaryIndex] = None, auto_create_table: bool = True):
 
-        self.fields_switch = dict()
+        self.fields_switch = FieldsSwitch()
         self._internal_mapping = dict()
         self._dynamodb_client = DynamoDbCoreAdapter(
             table_name=table_name, region_name=region_name,
@@ -78,7 +93,6 @@ class BaseTable:
         else:
             return False
 
-
     def get_single_field_item_from_single_item(self, key_name: str, key_value: str, field_to_get: str, query_kwargs: Optional[dict] = None) -> Any:
         response_data = self.dynamodb_client.get_item_in_path_target(
             key_name=key_name, key_value=key_value,
@@ -97,7 +111,6 @@ class BaseTable:
         )
         return response_data
 
-
     def _getters_to_database_paths(self, getters: Dict[str, FieldGetter]) -> Dict[str, List[DatabasePathElement]]:
         getters_database_paths: Dict[str, List[DatabasePathElement]] = dict()
         for getter_key, getter_item in getters.items():
@@ -120,7 +133,6 @@ class BaseTable:
         )
         return response_data
 
-
     def query(self, key_name: str, key_value: str, fields_to_get: List[str], index_name: Optional[str] = None, limit: Optional[int] = None,
               query_kwargs: Optional[dict] = None, filter_expression: Optional[Any] = None, **additional_kwargs) -> Optional[List[Any]]:
         # rendered_fields_paths = make_rendered_fields_paths(fields_paths=fields_to_get, query_kwargs=query_kwargs)
@@ -140,12 +152,11 @@ class BaseTable:
                         if matching_field_path_object is not None:
                             if matching_field_path_object.database_path is not None:
                                 matching_field_path_object.populate(value=current_item_value)
-                                current_item[current_item_key], valid = matching_field_path_object.validate_data(load_data_into_objects=False)
+                                current_item[current_item_key], valid = matching_field_path_object.validate_data()
             return response.items
         else:
             return None
 
-
     def set_update_one_field(self, key_name: str, key_value: str, target_field: str, value_to_set: Any,
                              index_name: Optional[str] = None, query_kwargs: Optional[dict] = None) -> bool:
         validated_data, valid, target_path_elements = process_validate_data_and_make_single_rendered_database_path(
@@ -198,7 +209,6 @@ class BaseTable:
         )
         return True if response is not None else False
 
-
     def remove_single_item_at_path_target(self, key_name: str, key_value: str, target_field: str,
                                           query_kwargs: Optional[dict] = None) -> bool:
         response: Optional[Response] = self.dynamodb_client.remove_data_elements_from_map(
@@ -230,7 +240,6 @@ class BaseTable:
             # return True, since technically, what needed to be performed (nothing) was performed.
             return True
 
-
     @property
     def internal_mapping(self) -> dict:
         return self._internal_mapping
@@ -254,7 +263,7 @@ def try_to_get_primitive_default_type_of_item(item_type: Any):
 
 
 def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optional[Any] = None, class_type: Optional[Any] = None,
-                                       nested_field_path: Optional[str] = None, current_path_elements: Optional[List[DatabasePathElement]] = None):
+                                       nested_field_path: Optional[str] = None, current_path_elements: Optional[List[DatabasePathElement]] = None, is_nested: Optional[bool] = False):
     if current_path_elements is None:
         current_path_elements = list()
     output_mapping = dict()
@@ -268,10 +277,14 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                         "to the assign_internal_mapping_from_class function"
             ))
 
+    # todo: re-implement some king of processed class types to avoid initializing
+    #  multiple times the same class when we have a nested class ?
     if class_type in table.processed_class_types:
-        return None
+        pass
+        # return None
     else:
-        table.processed_class_types.update({class_type})
+        pass
+        # table.processed_class_types.update({class_type})
 
     class_variables = class_type.__dict__
     required_fields = list()
@@ -308,12 +321,12 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                     required_fields.append(variable_item)
 
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
-                table.fields_switch[current_field_path] = variable_item
-
-                output_mapping[variable_item.field_name] = assign_internal_mapping_from_class(
-                    table=table, class_type=variable_item.map_model, nested_field_path=current_field_path,
-                    current_path_elements=[*variable_item.database_path]
-                )
+                field_is_valid = table.fields_switch.set(key=current_field_path, item=variable_item)
+                if field_is_valid is True:
+                    output_mapping[variable_item.field_name] = assign_internal_mapping_from_class(
+                        table=table, class_type=variable_item.map_model, nested_field_path=current_field_path,
+                        current_path_elements=[*variable_item.database_path]
+                    )
 
             elif isinstance(variable_item, BaseField):
                 variable_item: BaseField
@@ -330,25 +343,65 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                     required_fields.append(variable_item)
 
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
-                table.fields_switch[current_field_path] = variable_item
+                field_is_valid = table.fields_switch.set(key=current_field_path, item=copy(variable_item))
+                if field_is_valid is True:
+                    if variable_item.dict_items_excepted_type is not None:
+                        from StructNoSQL import ActiveSelf
+                        if variable_item.dict_items_excepted_type is ActiveSelf:
+                            variable_item._dict_items_excepted_type = class_type
 
-                if variable_item.dict_items_excepted_type is not None:
-                    current_field_path += ".{{" + variable_item.key_name + "}}"
+                        item_default_type = try_to_get_primitive_default_type_of_item(item_type=variable_item.dict_items_excepted_type)
+                        item_key_name = make_dict_key_var_name(key_name=variable_item.key_name)
 
-                    item_default_type = try_to_get_primitive_default_type_of_item(item_type=variable_item.dict_items_excepted_type)
-                    map_item = MapItem(
-                        parent_field=variable_item, field_type=item_default_type,
-                        model_type=variable_item.dict_items_excepted_type
-                    )
-                    table.fields_switch[current_field_path] = map_item
-                    item_key_name = make_dict_key_var_name(key_name=variable_item.key_name)
+                        if "{i}" in variable_item.key_name:
+                            if is_nested is not True:
+                                current_nested_field_path = "" if nested_field_path is None else f"{nested_field_path}"
+                                current_nested_database_path = [*variable_item.database_path]
+                                for i in range(variable_item.max_nested):
+                                    nested_variable_item = variable_item.copy()
+                                    nested_variable_item._database_path = [*current_nested_database_path]
+                                    item_rendered_key_name = nested_variable_item.key_name.replace("{i}", f"{i}")
 
-                    if variable_item.dict_items_excepted_type not in PRIMITIVE_TYPES:
-                        new_database_dict_item_path_element = DatabasePathElement(element_key=item_key_name, default_type=item_default_type)
-                        output_mapping[item_key_name] = assign_internal_mapping_from_class(
-                            table=table, class_type=variable_item.dict_items_excepted_type, nested_field_path=current_field_path,
-                            current_path_elements=[*variable_item.database_path, new_database_dict_item_path_element]
-                        )
+                                    map_item = MapItem(
+                                        parent_field=nested_variable_item,
+                                        field_type=nested_variable_item.default_field_type,
+                                        model_type=nested_variable_item.dict_items_excepted_type
+                                    )
+                                    current_nested_field_path += f".{variable_item.field_name}"
+                                    current_nested_field_path += ".{{" + item_rendered_key_name + "}}"
+
+                                    current_nested_database_path.append(DatabasePathElement(
+                                        element_key=make_dict_key_var_name(item_rendered_key_name),
+                                        default_type=nested_variable_item.default_field_type,
+                                        custom_default_value=nested_variable_item.custom_default_value
+                                    ))
+                                    field_is_valid = table.fields_switch.set(key=current_nested_field_path, item=map_item)
+                                    if field_is_valid is True:
+                                        if variable_item.dict_items_excepted_type not in PRIMITIVE_TYPES:
+                                            output_mapping[item_key_name] = assign_internal_mapping_from_class(
+                                                table=table, class_type=variable_item.dict_items_excepted_type,
+                                                nested_field_path=current_nested_field_path,
+                                                current_path_elements=[*current_nested_database_path], is_nested=True
+                                            )
+                                    current_nested_database_path.append(DatabasePathElement(
+                                        element_key=nested_variable_item.field_name,
+                                        default_type=nested_variable_item.default_field_type,
+                                        custom_default_value=nested_variable_item.custom_default_value
+                                    ))
+                        else:
+                            current_field_path += ".{{" + variable_item.key_name + "}}"
+                            map_item = MapItem(
+                                parent_field=variable_item, field_type=item_default_type,
+                                model_type=variable_item.dict_items_excepted_type
+                            )
+                            field_is_valid = table.fields_switch.set(current_field_path, map_item)
+                            if field_is_valid is True:
+                                if variable_item.dict_items_excepted_type not in PRIMITIVE_TYPES:
+                                    new_database_dict_item_path_element = DatabasePathElement(element_key=item_key_name, default_type=item_default_type)
+                                    output_mapping[item_key_name] = assign_internal_mapping_from_class(
+                                        table=table, class_type=variable_item.dict_items_excepted_type, nested_field_path=current_field_path,
+                                        current_path_elements=[*variable_item.database_path, new_database_dict_item_path_element]
+                                    )
 
         except Exception as e:
             print(e)
